<?xml version="1.0" encoding="utf-8"?>

<!-- Original Japanese translation done by omiya tou (tokyogeometry / github) -->

<!DOCTYPE TS>
<TS version="2.1" language="ja">
<context>
    <name>ColorParameter</name>
    <message>
        <location filename="../src/FilterParameters/ColorParameter.cpp" line="132"/>
        <source>Select color</source>
        <translation>色を選択</translation>
    </message>
</context>
<context>
    <name>DialogSettings</name>
    <message>
        <location filename="../ui/dialogsettings.ui" line="14"/>
        <source>Dialog</source>
        <translation>ダイアログ</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="102"/>
        <source>Internet updates</source>
        <translation>インターネット更新</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="111"/>
        <source>Update now</source>
        <translation>今すぐ更新</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="140"/>
        <source>Layout</source>
        <translation>レイアウト</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="204"/>
        <source>Preview on the &amp;left</source>
        <translation>左側にプレビューを表示(&amp;L)</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="167"/>
        <source>Pre&amp;view on right side</source>
        <translation>右側にプレビューを表示(&amp;V)</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="24"/>
        <source>Theme</source>
        <translation>テーマ</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="30"/>
        <source>&amp;Default</source>
        <translation>デフォルト(&amp;D)</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="37"/>
        <source>Dar&amp;k</source>
        <translation>暗い色(&amp;K)</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="50"/>
        <source>&lt;i&gt;(Restart needed)&lt;/I&gt;</source>
        <translation>&lt;i&gt;(再起動が必要です)&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="60"/>
        <source>Language</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="72"/>
        <location filename="../src/DialogSettings.cpp" line="77"/>
        <source>Output messages</source>
        <translation>出力メッセージ</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="84"/>
        <source>Dialogs</source>
        <translation>ダイアログ</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="90"/>
        <source>Use native color dialog</source>
        <translation>ネイティブな色選択ダイアログを使用</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="121"/>
        <source>Preview</source>
        <translation>プレビュー</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="127"/>
        <source>Timeout (seconds)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="176"/>
        <source>Show institution logos</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="240"/>
        <source>&amp;Ok</source>
        <translation>OK(&amp;O)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="57"/>
        <source>Settings</source>
        <translation>設定</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="63"/>
        <source>Never</source>
        <translation>行わない</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="64"/>
        <source>Daily</source>
        <translation>毎日</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="65"/>
        <source>Weekly</source>
        <translation>毎週</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="66"/>
        <source>Every 2 weeks</source>
        <translation>隔週</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="67"/>
        <source>Monthly</source>
        <translation>毎月</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="69"/>
        <source>At launch (debug)</source>
        <translation>起動時 (デバッグ用)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="78"/>
        <source>Output messages...</source>
        <translation>出力メッセージ...</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="79"/>
        <source>Quiet (default)</source>
        <translation>なし (デフォルト)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="80"/>
        <source>Verbose (layer name)</source>
        <translation>詳細 (レイヤー名)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="81"/>
        <source>Verbose (console)</source>
        <translation>詳細 (コンソール)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="82"/>
        <source>Verbose (log file)</source>
        <translation>詳細 (ログファイル)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="83"/>
        <source>Very verbose (console)</source>
        <translation>非常に詳細 (コンソール)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="84"/>
        <source>Very verbose (log file)</source>
        <translation>非常に詳細 (ログファイル)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="85"/>
        <source>Debug (console)</source>
        <translation>デバッグ (コンソール)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="86"/>
        <source>Debug (log file)</source>
        <translation>デバッグ (ログファイル)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="102"/>
        <source>Check to use Native/OS color dialog, uncheck to use Qt&apos;s</source>
        <translation>チェックすると、システムに標準搭載された色選択ダイアログを使用します。チェックを外すと、Qt の色選択ダイアログを使用します</translation>
    </message>
</context>
<context>
    <name>FileParameter</name>
    <message>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="140"/>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="143"/>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="146"/>
        <source>Select a file</source>
        <translation>ファイルを選択</translation>
    </message>
</context>
<context>
    <name>FilterParametersWidget</name>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="38"/>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="147"/>
        <source>&lt;i&gt;Select a filter&lt;/i&gt;</source>
        <translation>&lt;i&gt;フィルタを選択&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="116"/>
        <source>&lt;i&gt;No parameters&lt;/i&gt;</source>
        <translation>&lt;i&gt;パラメータなし&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="121"/>
        <source>Error parsing filter parameters

</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FiltersView</name>
    <message>
        <location filename="../ui/filtersview.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="64"/>
        <source>Rename fave</source>
        <translation>お気に入りの名前を変更</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="66"/>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="332"/>
        <source>Remove fave</source>
        <translation>お気に入りから削除</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="68"/>
        <source>Clone fave</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="72"/>
        <source>Add fave</source>
        <translation>お気に入りに追加</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="332"/>
        <source>Do you really want to remove the following fave?

%1
</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FolderParameter</name>
    <message>
        <location filename="../src/FilterParameters/FolderParameter.cpp" line="118"/>
        <source>Select a folder</source>
        <translation>フォルダを選択</translation>
    </message>
</context>
<context>
    <name>HeadlessProgressDialog</name>
    <message>
        <location filename="../ui/headlessprogressdialog.ui" line="14"/>
        <source>Dialog</source>
        <translation>ダイアログ</translation>
    </message>
    <message>
        <location filename="../ui/headlessprogressdialog.ui" line="68"/>
        <source>Cancel</source>
        <translation>キャンセル</translation>
    </message>
</context>
<context>
    <name>InOutPanel</name>
    <message>
        <location filename="../ui/inoutpanel.ui" line="40"/>
        <source>Input / Output</source>
        <translation>入力 / 出力</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="47"/>
        <source>...</source>
        <translation>...</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="112"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="44"/>
        <source>Input layers</source>
        <translation>入力レイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="45"/>
        <source>None</source>
        <translation>なし</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="46"/>
        <source>Active (default)</source>
        <translation>アクティブなレイヤー (デフォルト)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="47"/>
        <source>All</source>
        <translation>すべて</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="48"/>
        <source>Active and below</source>
        <translation>アクティブなレイヤーとその下のレイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="49"/>
        <source>Active and above</source>
        <translation>アクティブなレイヤーとその上のレイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="50"/>
        <source>All visible</source>
        <translation>すべての可視レイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="51"/>
        <source>All invisible</source>
        <translation>すべての不可視レイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="52"/>
        <source>All visible (decr.)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="53"/>
        <source>All invisible (decr.)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="54"/>
        <source>All (decr.)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="158"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="56"/>
        <source>Output mode</source>
        <translation>出力モード</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="57"/>
        <source>In place (default)</source>
        <translation>現在のレイヤー (デフォルト)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="58"/>
        <source>New layer(s)</source>
        <translation>新規レイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="59"/>
        <source>New active layer(s)</source>
        <translation>新規アクティブレイヤー</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="60"/>
        <source>New image</source>
        <translation>新規画像</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="89"/>
        <source>Output messages</source>
        <translation>出力メッセージ</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="135"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="62"/>
        <source>Preview mode</source>
        <translation>プレビューモード</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="63"/>
        <source>1st output (default)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="64"/>
        <source>2nd output</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="65"/>
        <source>3rd output</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="66"/>
        <source>4th output</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="67"/>
        <source>1st -&gt; 2nd output</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="68"/>
        <source>1st -&gt; 3rd output</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="69"/>
        <source>1st -&gt; 4th output</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="70"/>
        <source>All outputs</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>LanguageSelectionWidget</name>
    <message>
        <location filename="../ui/languageselectionwidget.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/languageselectionwidget.ui" line="38"/>
        <source>&lt;i&gt;(Restart needed)&lt;/i&gt;</source>
        <translation>&lt;i&gt;(再起動が必要です)&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/Widgets/LanguageSelectionWidget.cpp" line="45"/>
        <source>System default (%1)</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>MainWindow</name>
    <message>
        <location filename="../ui/mainwindow.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="155"/>
        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Download filter definitions from remote sources&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
        <translation>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Télécharger les définitions de filtres depuis les sources distantes&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="158"/>
        <source>Internet</source>
        <translation>インターネット</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="104"/>
        <source>Selection mode</source>
        <translation>選択モード</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="532"/>
        <source>TextLabel</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="227"/>
        <source>...</source>
        <translation>...</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="370"/>
        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Enable/disable preview&lt;br/&gt;(right click on preview image for instant swaping)&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
        <translation>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Activer/désactiver l&apos;aperçu&lt;br/&gt;(clic droit sur l&apos;aperçu pour alterner)&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="373"/>
        <source>Preview</source>
        <translation>プレビュー</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="516"/>
        <source>Settings...</source>
        <translation>設定...</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="545"/>
        <source>&amp;Cancel</source>
        <translation>キャンセル(&amp;C)</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="558"/>
        <source>&amp;Fullscreen</source>
        <translation>フルスクリーン(&amp;F)</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="571"/>
        <source>&amp;Apply</source>
        <translation>適用(&amp;A)</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="584"/>
        <source>&amp;OK</source>
        <translation>OK(&amp;O)</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="77"/>
        <source>Zoom in</source>
        <translation>ズームイン</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="78"/>
        <source>Zoom out</source>
        <translation>ズームアウト</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="79"/>
        <source>Reset zoom</source>
        <translation>拡大率をリセット</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="81"/>
        <source>Warning: Preview may be inaccurate (zoom factor has been modified)</source>
        <translation>警告: プレビューは実際の処理結果と異なる場合があります (拡大率が変更されています)</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="82"/>
        <source>Add fave</source>
        <translation>お気に入りに追加</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="84"/>
        <source>Reset parameters to default values</source>
        <translation>パラメータの値を初期化</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="87"/>
        <source>Update filters (Ctrl+R / F5)</source>
        <translation>フィルタを更新</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="89"/>
        <source>Rename fave</source>
        <translation>お気に入りの名前を変更</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="91"/>
        <source>Remove fave</source>
        <translation>お気に入りから削除</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="95"/>
        <source>Expand/Collapse all</source>
        <translation>すべて展開/すべて畳む</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="97"/>
        <source>G&apos;MIC (http://gmic.eu)&lt;br/&gt;GREYC (http://www.greyc.fr)&lt;br/&gt;CNRS (http://www.cnrs.fr)&lt;br/&gt;Normandy University (http://www.unicaen.fr)&lt;br/&gt;Ensicaen (http://www.ensicaen.fr)</source>
        <translation>G&apos;MIC (http://gmic.eu)&lt;br/&gt;GREYC (http://www.greyc.fr)&lt;br/&gt;フランス国立科学研究センター (http://www.cnrs.fr)&lt;br/&gt;カーン・ノルマンディー大学 (http://www.unicaen.fr)&lt;br/&gt;Ensicaen (http://www.ensicaen.fr)</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="263"/>
        <source>Update completed</source>
        <translation>更新が完了しました</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="263"/>
        <location filename="../src/MainWindow.cpp" line="265"/>
        <location filename="../src/MainWindow.cpp" line="316"/>
        <source>Filter definitions have been updated.</source>
        <translation>フィルタ定義の更新が完了しました。</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="268"/>
        <source>No download was needed.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="447"/>
        <source>The update could not be achieved&lt;br&gt;because of the following errors:&lt;br&gt;</source>
        <translation>以下のエラーにより&lt;br/&gt;更新に失敗しました:&lt;br/&gt;</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="453"/>
        <source>Update error</source>
        <translation>更新エラー</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="596"/>
        <source>Error</source>
        <translation>エラー</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="675"/>
        <source>Waiting for cancelled jobs...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1004"/>
        <source>Import faves</source>
        <translation>お気に入りをインポート</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1004"/>
        <source>Do you want to import faves from file below?&lt;br/&gt;%1</source>
        <translation>以下のファイルからお気に入りをインポートしますか？&lt;br/&gt;%1</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1007"/>
        <source>Don&apos;t ask again</source>
        <translation>次回から確認しない</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1096"/>
        <source>Confirmation</source>
        <translation>確認</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1096"/>
        <source>A gmic command is running.&lt;br&gt;Do you really want to close the plugin?</source>
        <translation>G&apos;MIC コマンドが実行中です。&lt;br&gt;本当にプラグインを終了しますか？</translation>
    </message>
</context>
<context>
    <name>MultilineTextParameterWidget</name>
    <message>
        <location filename="../ui/multilinetextparameterwidget.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/multilinetextparameterwidget.ui" line="29"/>
        <source>Update</source>
        <translation>更新</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/MultilineTextParameterWidget.cpp" line="38"/>
        <source>Ctrl+Return</source>
        <translation>Ctrl+Enter</translation>
    </message>
</context>
<context>
    <name>ProgressInfoWidget</name>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="48"/>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="46"/>
        <source>Abort</source>
        <translation>中止</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="55"/>
        <source>TextLabel</source>
        <translation>TextLabel</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="43"/>
        <source>G&apos;MIC-Qt Plug-in progression</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="141"/>
        <source>Updating filters...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="196"/>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="209"/>
        <source>[Processing %1 | %2]</source>
        <translation>[実行中: %1 | %2]</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="211"/>
        <source>[Processing %1]</source>
        <translation>[実行中: %1]</translation>
    </message>
</context>
<context>
    <name>ProgressInfoWindow</name>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="14"/>
        <source>MainWindow</source>
        <translation>MainWindow</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="30"/>
        <location filename="../ui/progressinfowindow.ui" line="50"/>
        <source>TextLabel</source>
        <translation>TextLabel</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="72"/>
        <source>Cancel</source>
        <translation>キャンセル</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="103"/>
        <source>%1 seconds</source>
        <translation>%1 秒</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="113"/>
        <source>[Processing %1 | %2]</source>
        <translation>|実行中: %1 | %2]</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="115"/>
        <source>[Processing %1]</source>
        <translation>|実行中: %1]</translation>
    </message>
</context>
<context>
    <name>QObject</name>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="81"/>
        <source>Select an image to open...</source>
        <translation>開く画像を選択...</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="81"/>
        <source>PNG &amp; JPG files (*.png *.jpeg *.jpg *.PNG *.JPEG *.JPG)</source>
        <translation>PNG および JPG ファイル (*.png *.jpeg *.jpg *.PNG *.JPEG *.JPG)</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="87"/>
        <source>Error</source>
        <translation>エラー</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="87"/>
        <source>Could not open file.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="91"/>
        <source>Default image</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="87"/>
        <source>Visible</source>
        <translation>一覧に表示</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersPresenter.cpp" line="85"/>
        <source>Available filters (%1)</source>
        <translation>使用可能なフィルタ (%1)</translation>
    </message>
</context>
<context>
    <name>SearchFieldWidget</name>
    <message>
        <location filename="../ui/SearchFieldWidget.ui" line="20"/>
        <source>Frame</source>
        <translation>フレーム</translation>
    </message>
    <message>
        <location filename="../src/Widgets/SearchFieldWidget.cpp" line="72"/>
        <source>Search</source>
        <translation>検索</translation>
    </message>
    <message>
        <location filename="../src/Widgets/SearchFieldWidget.cpp" line="73"/>
        <source>Search in filters list (%1)</source>
        <translation>フィルタ一覧を検索 (%1)</translation>
    </message>
</context>
<context>
    <name>Updater</name>
    <message>
        <location filename="../src/Updater.cpp" line="206"/>
        <source>Could not read/decompress %1</source>
        <translation>%1 を読み込み・展開できませんでした</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="197"/>
        <source>Error downloading %1 (empty file?)</source>
        <translation>%1 のダウンロード中にエラーが発生しました (空ファイル？)</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="212"/>
        <source>Error creating file %1</source>
        <translation>ファイル %1 の作成時にエラーが発生しました</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="216"/>
        <source>Error writing file %1</source>
        <translation>ファイル %1 の書き込み中にエラーが発生しました</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="228"/>
        <source>Error downloading %1</source>
        <translation>%1 のダウンロード中にエラーが発生しました</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="256"/>
        <source>Download timeout: %1</source>
        <translation>ダウンロード中にタイムアウト: %1</translation>
    </message>
</context>
<context>
    <name>ZoomLevelSelector</name>
    <message>
        <location filename="../ui/zoomlevelselector.ui" line="14"/>
        <source>Form</source>
        <translation>フォーム</translation>
    </message>
</context>
</TS>
