<?xml version="1.0" encoding="utf-8"?>

<!-- Original Czech translation done by Jan Helebrant -->

<!DOCTYPE TS>
<TS version="2.1" language="cs_CZ">
<context>
    <name>ColorParameter</name>
    <message>
        <location filename="../src/FilterParameters/ColorParameter.cpp" line="132"/>
        <source>Select color</source>
        <translation>Vyberte barvu</translation>
    </message>
</context>
<context>
    <name>DialogSettings</name>
    <message>
        <location filename="../ui/dialogsettings.ui" line="14"/>
        <source>Dialog</source>
        <translation>Dialog</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="102"/>
        <source>Internet updates</source>
        <translation>Online aktualizace</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="111"/>
        <source>Update now</source>
        <translation>Aktualizovat nyní</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="140"/>
        <source>Layout</source>
        <translation>Rozvržení</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="204"/>
        <source>Preview on the &amp;left</source>
        <translation>Náhled &amp;vlevo</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="167"/>
        <source>Pre&amp;view on right side</source>
        <translation>Náhled &amp;vpravo</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="24"/>
        <source>Theme</source>
        <translation>Téma</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="30"/>
        <source>&amp;Default</source>
        <translation>&amp;Výchozí</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="37"/>
        <source>Dar&amp;k</source>
        <translation>&amp;Tmavé</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="50"/>
        <source>&lt;i&gt;(Restart needed)&lt;/I&gt;</source>
        <translation>&lt;i&gt;(vyžaduje restart)&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="60"/>
        <source>Language</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="72"/>
        <location filename="../src/DialogSettings.cpp" line="77"/>
        <source>Output messages</source>
        <translation type="unfinished">Výstupní zprávy</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="84"/>
        <source>Dialogs</source>
        <translation>Dialogy</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="90"/>
        <source>Use native color dialog</source>
        <translation>Použít nativní dialog barev</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="121"/>
        <source>Preview</source>
        <translation type="unfinished">Náhled</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="127"/>
        <source>Timeout (seconds)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="176"/>
        <source>Show institution logos</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="240"/>
        <source>&amp;Ok</source>
        <translation>&amp;Ok</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="57"/>
        <source>Settings</source>
        <translation>Nastavení</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="63"/>
        <source>Never</source>
        <translation>Nikdy</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="64"/>
        <source>Daily</source>
        <translation>Denně</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="65"/>
        <source>Weekly</source>
        <translation>Týdně</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="66"/>
        <source>Every 2 weeks</source>
        <translation>Každé 2 týdny</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="67"/>
        <source>Monthly</source>
        <translation>Měsíčně</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="69"/>
        <source>At launch (debug)</source>
        <translation>Při startu (ladění)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="78"/>
        <source>Output messages...</source>
        <translation>Výstupní zprávy...</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="79"/>
        <source>Quiet (default)</source>
        <translation>Tichý (výchozí)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="80"/>
        <source>Verbose (layer name)</source>
        <translation>Upovídaný (název vrstvy)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="81"/>
        <source>Verbose (console)</source>
        <translation>Upovídaný (konzole)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="82"/>
        <source>Verbose (log file)</source>
        <translation>Upovídaný (soubor záznamů)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="83"/>
        <source>Very verbose (console)</source>
        <translation>Velmi upovídaný (konzole)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="84"/>
        <source>Very verbose (log file)</source>
        <translation>Velmi upovídaný (soubor záznamů)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="85"/>
        <source>Debug (console)</source>
        <translation>Ladění (konzole)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="86"/>
        <source>Debug (log file)</source>
        <translation>Ladění (soubor záznamů)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="102"/>
        <source>Check to use Native/OS color dialog, uncheck to use Qt&apos;s</source>
        <translation>Zaškrtněte pro použití nativního/OS dialogu barev, odškrtněte pro použití Qt dialogu</translation>
    </message>
</context>
<context>
    <name>FileParameter</name>
    <message>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="140"/>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="143"/>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="146"/>
        <source>Select a file</source>
        <translation>Vyberte soubor</translation>
    </message>
</context>
<context>
    <name>FilterParametersWidget</name>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="38"/>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="147"/>
        <source>&lt;i&gt;Select a filter&lt;/i&gt;</source>
        <translation>&lt;i&gt;Vyberte filtr&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="116"/>
        <source>&lt;i&gt;No parameters&lt;/i&gt;</source>
        <translation>&lt;i&gt;Žádné parametry&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="121"/>
        <source>Error parsing filter parameters

</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FiltersView</name>
    <message>
        <location filename="../ui/filtersview.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished">GMIC</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="64"/>
        <source>Rename fave</source>
        <translation>Přejmenovat oblíbený</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="66"/>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="332"/>
        <source>Remove fave</source>
        <translation>Odstranit oblíbený</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="68"/>
        <source>Clone fave</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="72"/>
        <source>Add fave</source>
        <translation>Přidat oblíbený</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="332"/>
        <source>Do you really want to remove the following fave?

%1
</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FolderParameter</name>
    <message>
        <location filename="../src/FilterParameters/FolderParameter.cpp" line="118"/>
        <source>Select a folder</source>
        <translation>Vyberte složku</translation>
    </message>
</context>
<context>
    <name>HeadlessProgressDialog</name>
    <message>
        <location filename="../ui/headlessprogressdialog.ui" line="14"/>
        <source>Dialog</source>
        <translation>Dialog</translation>
    </message>
    <message>
        <location filename="../ui/headlessprogressdialog.ui" line="68"/>
        <source>Cancel</source>
        <translation>Storno</translation>
    </message>
</context>
<context>
    <name>InOutPanel</name>
    <message>
        <location filename="../ui/inoutpanel.ui" line="40"/>
        <source>Input / Output</source>
        <translation>Vstup / Výstup</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="47"/>
        <source>...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="112"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="44"/>
        <source>Input layers</source>
        <translation>Vstupní vrstvy</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="45"/>
        <source>None</source>
        <translation>Žádné</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="46"/>
        <source>Active (default)</source>
        <translation>Aktivní (výchozí)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="47"/>
        <source>All</source>
        <translation>Vše</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="48"/>
        <source>Active and below</source>
        <translation>Aktivní a níže</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="49"/>
        <source>Active and above</source>
        <translation>Aktivní a výše</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="50"/>
        <source>All visible</source>
        <translation>Všechny viditelné</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="51"/>
        <source>All invisible</source>
        <translation>Všechny neviditelné</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="52"/>
        <source>All visible (decr.)</source>
        <translation>Všechny viditelné (sestup.)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="53"/>
        <source>All invisible (decr.)</source>
        <translation>Všechny neviditelné (sestup.)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="54"/>
        <source>All (decr.)</source>
        <translation>Všechny (sestup.)</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="158"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="56"/>
        <source>Output mode</source>
        <translation>Režim výstupu</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="57"/>
        <source>In place (default)</source>
        <translation>V místě (výchozí)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="58"/>
        <source>New layer(s)</source>
        <translation>Nová vrstva(y)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="59"/>
        <source>New active layer(s)</source>
        <translation>Nová aktivní vrstva(y)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="60"/>
        <source>New image</source>
        <translation>Nový obrázek</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="89"/>
        <source>Output messages</source>
        <translation>Výstupní zprávy</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="135"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="62"/>
        <source>Preview mode</source>
        <translation>Režim náhledu</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="63"/>
        <source>1st output (default)</source>
        <translation>1. výstup (výchozí)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="64"/>
        <source>2nd output</source>
        <translation>2. výstup</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="65"/>
        <source>3rd output</source>
        <translation>3. výstup</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="66"/>
        <source>4th output</source>
        <translation>4. výstup</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="67"/>
        <source>1st -&gt; 2nd output</source>
        <translation>1. -&gt; 2. výstup</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="68"/>
        <source>1st -&gt; 3rd output</source>
        <translation>1. -&gt; 3. výstup</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="69"/>
        <source>1st -&gt; 4th output</source>
        <translation>1. -&gt; 4. výstup</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="70"/>
        <source>All outputs</source>
        <translation>Všechny výstupy</translation>
    </message>
</context>
<context>
    <name>LanguageSelectionWidget</name>
    <message>
        <location filename="../ui/languageselectionwidget.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished">GMIC</translation>
    </message>
    <message>
        <location filename="../ui/languageselectionwidget.ui" line="38"/>
        <source>&lt;i&gt;(Restart needed)&lt;/i&gt;</source>
        <translation>&lt;i&gt;(Vyžaduje restart)&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/Widgets/LanguageSelectionWidget.cpp" line="45"/>
        <source>System default (%1)</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>MainWindow</name>
    <message>
        <location filename="../ui/mainwindow.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="155"/>
        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Download filter definitions from remote sources&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
        <translation>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Stáhnout definice filtrů ze vzdálených zdrojů&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="158"/>
        <source>Internet</source>
        <translation>Internet</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="104"/>
        <source>Selection mode</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="532"/>
        <source>TextLabel</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="227"/>
        <source>...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="370"/>
        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Enable/disable preview&lt;br/&gt;(right click on preview image for instant swaping)&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
        <translation>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Zapnout/vypnout náhled&lt;br/&gt;(klik pravým tlačítkem myši na náhled pro přepnutí)&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="373"/>
        <source>Preview</source>
        <translation>Náhled</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="516"/>
        <source>Settings...</source>
        <translation>Nastavení...</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="545"/>
        <source>&amp;Cancel</source>
        <translation>&amp;Storno</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="558"/>
        <source>&amp;Fullscreen</source>
        <translation>&amp;Celá obrazovka</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="571"/>
        <source>&amp;Apply</source>
        <translation>&amp;Použít</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="584"/>
        <source>&amp;OK</source>
        <translation>&amp;Ok</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="77"/>
        <source>Zoom in</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="78"/>
        <source>Zoom out</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="79"/>
        <source>Reset zoom</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="81"/>
        <source>Warning: Preview may be inaccurate (zoom factor has been modified)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="82"/>
        <source>Add fave</source>
        <translation>Přidat oblíbený</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="84"/>
        <source>Reset parameters to default values</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="87"/>
        <source>Update filters (Ctrl+R / F5)</source>
        <translation>Aktualizovat filtry (Ctrl+R / F5)</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="89"/>
        <source>Rename fave</source>
        <translation>Přejmenovat oblíbený</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="91"/>
        <source>Remove fave</source>
        <translation>Odstranit oblíbený</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="95"/>
        <source>Expand/Collapse all</source>
        <translation>Rozbalit/Sbalit vše</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="97"/>
        <source>G&apos;MIC (http://gmic.eu)&lt;br/&gt;GREYC (http://www.greyc.fr)&lt;br/&gt;CNRS (http://www.cnrs.fr)&lt;br/&gt;Normandy University (http://www.unicaen.fr)&lt;br/&gt;Ensicaen (http://www.ensicaen.fr)</source>
        <translation>G&apos;MIC (http://gmic.eu)&lt;br/&gt;GREYC (http://www.greyc.fr)&lt;br/&gt;CNRS (http://www.cnrs.fr)&lt;br/&gt;Normandy University (http://www.unicaen.fr)&lt;br/&gt;Ensicaen (http://www.ensicaen.fr)</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="263"/>
        <source>Update completed</source>
        <translation>Aktualizace dokončena</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="263"/>
        <location filename="../src/MainWindow.cpp" line="265"/>
        <location filename="../src/MainWindow.cpp" line="316"/>
        <source>Filter definitions have been updated.</source>
        <translation>Definice filtrů byly aktualizovány.</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="268"/>
        <source>No download was needed.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="447"/>
        <source>The update could not be achieved&lt;br&gt;because of the following errors:&lt;br&gt;</source>
        <translation>Aktualizace nemohla být provedena&lt;br/&gt;kvůli následujícím chybám:&lt;br/&gt;</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="453"/>
        <source>Update error</source>
        <translation>Chyba aktualizace</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="596"/>
        <source>Error</source>
        <translation>Chyba</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="675"/>
        <source>Waiting for cancelled jobs...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1004"/>
        <source>Import faves</source>
        <translation>Importovat oblíbené</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1004"/>
        <source>Do you want to import faves from file below?&lt;br/&gt;%1</source>
        <translation>Chcete importovat oblíbené ze souboru níže?&lt;br/&gt;%1</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1007"/>
        <source>Don&apos;t ask again</source>
        <translation>Znovu už se neptat</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1096"/>
        <source>Confirmation</source>
        <translation>Potvrzení</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1096"/>
        <source>A gmic command is running.&lt;br&gt;Do you really want to close the plugin?</source>
        <translation>Gmic příkaz stále běží.&lt;br&gt;Opravdu chcete zavřít plugin?</translation>
    </message>
</context>
<context>
    <name>MultilineTextParameterWidget</name>
    <message>
        <location filename="../ui/multilinetextparameterwidget.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/multilinetextparameterwidget.ui" line="29"/>
        <source>Update</source>
        <translation>Aktualizovat</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/MultilineTextParameterWidget.cpp" line="38"/>
        <source>Ctrl+Return</source>
        <translation>Ctrl+Enter</translation>
    </message>
</context>
<context>
    <name>ProgressInfoWidget</name>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="48"/>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="46"/>
        <source>Abort</source>
        <translation>Zrušit</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="55"/>
        <source>TextLabel</source>
        <translation>TextLabel</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="43"/>
        <source>G&apos;MIC-Qt Plug-in progression</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="141"/>
        <source>Updating filters...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="196"/>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="209"/>
        <source>[Processing %1 | %2]</source>
        <translation>[Zpracování %1 | %2]</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="211"/>
        <source>[Processing %1]</source>
        <translation>[Zpracování %1]</translation>
    </message>
</context>
<context>
    <name>ProgressInfoWindow</name>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="14"/>
        <source>MainWindow</source>
        <translation>MainWindow</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="30"/>
        <location filename="../ui/progressinfowindow.ui" line="50"/>
        <source>TextLabel</source>
        <translation>TextLabel</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="72"/>
        <source>Cancel</source>
        <translation>Storno</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="103"/>
        <source>%1 seconds</source>
        <translation>%1 sekund</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="113"/>
        <source>[Processing %1 | %2]</source>
        <translation>[Zpracování %1 | %2]</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="115"/>
        <source>[Processing %1]</source>
        <translation>[Zpracování %1]</translation>
    </message>
</context>
<context>
    <name>QObject</name>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="81"/>
        <source>Select an image to open...</source>
        <translation>Vyberte obrázek k otevření...</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="81"/>
        <source>PNG &amp; JPG files (*.png *.jpeg *.jpg *.PNG *.JPEG *.JPG)</source>
        <translation>Soubory PNG &amp; JPG (*.png *.jpeg *.jpg *.PNG *.JPEG *.JPG)</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="87"/>
        <source>Error</source>
        <translation type="unfinished">Chyba</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="87"/>
        <source>Could not open file.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="91"/>
        <source>Default image</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="87"/>
        <source>Visible</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersPresenter.cpp" line="85"/>
        <source>Available filters (%1)</source>
        <translation>Dostupných filtrů (%1)</translation>
    </message>
</context>
<context>
    <name>SearchFieldWidget</name>
    <message>
        <location filename="../ui/SearchFieldWidget.ui" line="20"/>
        <source>Frame</source>
        <translation>Frame</translation>
    </message>
    <message>
        <location filename="../src/Widgets/SearchFieldWidget.cpp" line="72"/>
        <source>Search</source>
        <translation>Hledat</translation>
    </message>
    <message>
        <location filename="../src/Widgets/SearchFieldWidget.cpp" line="73"/>
        <source>Search in filters list (%1)</source>
        <translation>Hledat v seznamu filtrů (%1)</translation>
    </message>
</context>
<context>
    <name>Updater</name>
    <message>
        <location filename="../src/Updater.cpp" line="206"/>
        <source>Could not read/decompress %1</source>
        <translation>Nemohu přečíst/dekomprimovat %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="197"/>
        <source>Error downloading %1 (empty file?)</source>
        <translation>Chyba stahování %1 (prázdný soubor?)</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="212"/>
        <source>Error creating file %1</source>
        <translation>Chyba vytváření souboru %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="216"/>
        <source>Error writing file %1</source>
        <translation>Chyba zapisování souboru %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="228"/>
        <source>Error downloading %1</source>
        <translation>Chyba stahování %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="256"/>
        <source>Download timeout: %1</source>
        <translation>Stahování zrušeno (časový limit překročen): %1</translation>
    </message>
</context>
<context>
    <name>ZoomLevelSelector</name>
    <message>
        <location filename="../ui/zoomlevelselector.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished">GMIC</translation>
    </message>
</context>
</TS>
