/** -*- mode: c++ ; c-basic-offset: 2 -*-
 *
 *  @file ConstParameter.cpp
 *
 *  Copyright 2017 Sebastien Fourey
 *
 *  This file is part of G'MIC-Qt, a generic plug-in for raster graphics
 *  editors, offering hundreds of filters thanks to the underlying G'MIC
 *  image processing framework.
 *
 *  gmic_qt is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  gmic_qt is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with gmic_qt.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "FilterParameters/ConstParameter.h"
#include <QString>
#include <QStringList>
#include "Common.h"
#include "HtmlTranslator.h"

ConstParameter::ConstParameter(QObject * parent) : AbstractParameter(parent, true) {}

ConstParameter::~ConstParameter() = default;

bool ConstParameter::isVisible() const
{
  return false;
}

void ConstParameter::addTo(QWidget *, int) {}

QString ConstParameter::textValue() const
{
  return _value;
}

void ConstParameter::setValue(const QString & value)
{
  _value = value;
}

void ConstParameter::reset()
{
  _value = _default;
}

bool ConstParameter::initFromText(const char * text, int & textLength)
{
  QStringList list = parseText("value", text, textLength);
  _name = HtmlTranslator::html2txt(list[0]);
  _value = _default = list[1];
  return true;
}
