<?xml version="1.0" encoding="utf-8"?>

<!-- Original Polish translation by Alex Mozheiko -->

<!DOCTYPE TS>
<TS version="2.1" language="pl">
<context>
    <name>ColorParameter</name>
    <message>
        <location filename="../src/FilterParameters/ColorParameter.cpp" line="132"/>
        <source>Select color</source>
        <translation>Wybór koloru</translation>
    </message>
</context>
<context>
    <name>DialogSettings</name>
    <message>
        <location filename="../ui/dialogsettings.ui" line="14"/>
        <source>Dialog</source>
        <translation>Okno dialogowe</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="102"/>
        <source>Internet updates</source>
        <translation>Aktualizacje z Internetu</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="111"/>
        <source>Update now</source>
        <translation>Uaktualnij teraz</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="140"/>
        <source>Layout</source>
        <translation>Układ</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="204"/>
        <source>Preview on the &amp;left</source>
        <translation>Podgląd &amp;po lewej stronie</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="167"/>
        <source>Pre&amp;view on right side</source>
        <translation>Podgląd &amp;po prawej stronie</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="24"/>
        <source>Theme</source>
        <translation>Wygląd</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="30"/>
        <source>&amp;Default</source>
        <translation>&amp;Domyślnie</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="37"/>
        <source>Dar&amp;k</source>
        <translation>&amp;Ciemny</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="50"/>
        <source>&lt;i&gt;(Restart needed)&lt;/I&gt;</source>
        <translation>&lt;i&gt;(Wymaga zrestartowania)&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="60"/>
        <source>Language</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="72"/>
        <location filename="../src/DialogSettings.cpp" line="77"/>
        <source>Output messages</source>
        <translation>Komunikat wyjścia</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="84"/>
        <source>Dialogs</source>
        <translation>Okna dialogowe</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="90"/>
        <source>Use native color dialog</source>
        <translation>Użyć kolorów systemowych</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="121"/>
        <source>Preview</source>
        <translation>Podgląd</translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="127"/>
        <source>Timeout (seconds)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="176"/>
        <source>Show institution logos</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/dialogsettings.ui" line="240"/>
        <source>&amp;Ok</source>
        <translation>&amp;Ok</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="57"/>
        <source>Settings</source>
        <translation>Ustawienia</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="63"/>
        <source>Never</source>
        <translation>Nigdy</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="64"/>
        <source>Daily</source>
        <translation>Codziennie</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="65"/>
        <source>Weekly</source>
        <translation>Cotygodniowo</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="66"/>
        <source>Every 2 weeks</source>
        <translation>Co 2 tygodnie</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="67"/>
        <source>Monthly</source>
        <translation>Miesięcznie</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="69"/>
        <source>At launch (debug)</source>
        <translation>Przy starcie (debugowanie)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="78"/>
        <source>Output messages...</source>
        <translation>Komunikat wyjścia...</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="79"/>
        <source>Quiet (default)</source>
        <translation>Brak (domyślnie)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="80"/>
        <source>Verbose (layer name)</source>
        <translation>Ogólny (z nazwą warstwy)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="81"/>
        <source>Verbose (console)</source>
        <translation>Ogólny (konsola)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="82"/>
        <source>Verbose (log file)</source>
        <translation>Ogólny (plik log)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="83"/>
        <source>Very verbose (console)</source>
        <translation>Dokładny (konsola)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="84"/>
        <source>Very verbose (log file)</source>
        <translation>Dokładny (plik log)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="85"/>
        <source>Debug (console)</source>
        <translation>Debugowanie (konsola)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="86"/>
        <source>Debug (log file)</source>
        <translation>Debugowanie (plik log)</translation>
    </message>
    <message>
        <location filename="../src/DialogSettings.cpp" line="102"/>
        <source>Check to use Native/OS color dialog, uncheck to use Qt&apos;s</source>
        <translation>Zaznacz aby używać natywny/systemowy zestaw kolorów, wyłącz dla QT</translation>
    </message>
</context>
<context>
    <name>FileParameter</name>
    <message>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="140"/>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="143"/>
        <location filename="../src/FilterParameters/FileParameter.cpp" line="146"/>
        <source>Select a file</source>
        <translation>Wybierz plik</translation>
    </message>
</context>
<context>
    <name>FilterParametersWidget</name>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="38"/>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="147"/>
        <source>&lt;i&gt;Select a filter&lt;/i&gt;</source>
        <translation>&lt;i&gt;Wybierz filtr&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="116"/>
        <source>&lt;i&gt;No parameters&lt;/i&gt;</source>
        <translation>&lt;i&gt;Brak parametrów&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/FilterParametersWidget.cpp" line="121"/>
        <source>Error parsing filter parameters

</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FiltersView</name>
    <message>
        <location filename="../ui/filtersview.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="64"/>
        <source>Rename fave</source>
        <translation>Zmiana nazwy ulubionego</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="66"/>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="332"/>
        <source>Remove fave</source>
        <translation>Usuń z ulubionych</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="68"/>
        <source>Clone fave</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="72"/>
        <source>Add fave</source>
        <translation>Dodaj do ulubionych</translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="332"/>
        <source>Do you really want to remove the following fave?

%1
</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FolderParameter</name>
    <message>
        <location filename="../src/FilterParameters/FolderParameter.cpp" line="118"/>
        <source>Select a folder</source>
        <translation>Wybierz katalog</translation>
    </message>
</context>
<context>
    <name>HeadlessProgressDialog</name>
    <message>
        <location filename="../ui/headlessprogressdialog.ui" line="14"/>
        <source>Dialog</source>
        <translation>Okno dialogowe</translation>
    </message>
    <message>
        <location filename="../ui/headlessprogressdialog.ui" line="68"/>
        <source>Cancel</source>
        <translation>Anuluj</translation>
    </message>
</context>
<context>
    <name>InOutPanel</name>
    <message>
        <location filename="../ui/inoutpanel.ui" line="40"/>
        <source>Input / Output</source>
        <translation>Wejście/Wyjście</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="47"/>
        <source>...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="112"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="44"/>
        <source>Input layers</source>
        <translation>Wartstwy wejściowe</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="45"/>
        <source>None</source>
        <translation>Brak</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="46"/>
        <source>Active (default)</source>
        <translation>Aktywna (domyślnie)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="47"/>
        <source>All</source>
        <translation>Wszystkie</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="48"/>
        <source>Active and below</source>
        <translation>Aktywna i poniżej</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="49"/>
        <source>Active and above</source>
        <translation>Aktywna i powyżej</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="50"/>
        <source>All visible</source>
        <translation>Wszystkie widoczne</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="51"/>
        <source>All invisible</source>
        <translation>Wszystkie niewidoczne</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="52"/>
        <source>All visible (decr.)</source>
        <translation>Wszystkie widoczne (kolejność malejąca)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="53"/>
        <source>All invisible (decr.)</source>
        <translation>Wszystkie niewidoczne (kolejność malejąca)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="54"/>
        <source>All (decr.)</source>
        <translation>Wszystkie (kolejność malejąca)</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="158"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="56"/>
        <source>Output mode</source>
        <translation>Tryb wyjścia</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="57"/>
        <source>In place (default)</source>
        <translation>Na miejscu (domyślnie)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="58"/>
        <source>New layer(s)</source>
        <translation>Nowa(e) warstwa(y)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="59"/>
        <source>New active layer(s)</source>
        <translation>Nowa(e) aktywna(e) warstwa(y)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="60"/>
        <source>New image</source>
        <translation>Nowy obraz</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="89"/>
        <source>Output messages</source>
        <translation>Komunikat wyjścia</translation>
    </message>
    <message>
        <location filename="../ui/inoutpanel.ui" line="135"/>
        <location filename="../src/Widgets/InOutPanel.cpp" line="62"/>
        <source>Preview mode</source>
        <translation>Tryb podglądu</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="63"/>
        <source>1st output (default)</source>
        <translation>1 wyjście (domyślnie)</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="64"/>
        <source>2nd output</source>
        <translation>2 wyjście</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="65"/>
        <source>3rd output</source>
        <translation>3 wyjście</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="66"/>
        <source>4th output</source>
        <translation>4 wyjście</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="67"/>
        <source>1st -&gt; 2nd output</source>
        <translation>od 1 do 2 wyjść</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="68"/>
        <source>1st -&gt; 3rd output</source>
        <translation>od 1 do 3 wyjść</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="69"/>
        <source>1st -&gt; 4th output</source>
        <translation>od 1 do 4 wyjść</translation>
    </message>
    <message>
        <location filename="../src/Widgets/InOutPanel.cpp" line="70"/>
        <source>All outputs</source>
        <translation>Wszystkie wyjścia</translation>
    </message>
</context>
<context>
    <name>LanguageSelectionWidget</name>
    <message>
        <location filename="../ui/languageselectionwidget.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished">GMIC</translation>
    </message>
    <message>
        <location filename="../ui/languageselectionwidget.ui" line="38"/>
        <source>&lt;i&gt;(Restart needed)&lt;/i&gt;</source>
        <translation>&lt;i&gt;(Wymaga zrestartowania)&lt;/i&gt;</translation>
    </message>
    <message>
        <location filename="../src/Widgets/LanguageSelectionWidget.cpp" line="45"/>
        <source>System default (%1)</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>MainWindow</name>
    <message>
        <location filename="../ui/mainwindow.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="155"/>
        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Download filter definitions from remote sources&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
        <translation>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Uaktualnienie filtrów przez internet&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="158"/>
        <source>Internet</source>
        <translation>Internet</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="104"/>
        <source>Selection mode</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="532"/>
        <source>TextLabel</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="227"/>
        <source>...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="370"/>
        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Enable/disable preview&lt;br/&gt;(right click on preview image for instant swaping)&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
        <translation>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Włącz/wyłącz podgląd&lt;br/&gt;(prawym przyciskiem na podgląd dla szybkiej zmiany)&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="373"/>
        <source>Preview</source>
        <translation>Podgląd</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="516"/>
        <source>Settings...</source>
        <translation>Okna dialogowe...</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="545"/>
        <source>&amp;Cancel</source>
        <translation>&amp;Anuluj</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="558"/>
        <source>&amp;Fullscreen</source>
        <translation>&amp;Pełny ekran</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="571"/>
        <source>&amp;Apply</source>
        <translation>&amp;Zastosuj</translation>
    </message>
    <message>
        <location filename="../ui/mainwindow.ui" line="584"/>
        <source>&amp;OK</source>
        <translation>&amp;Ok</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="77"/>
        <source>Zoom in</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="78"/>
        <source>Zoom out</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="79"/>
        <source>Reset zoom</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="81"/>
        <source>Warning: Preview may be inaccurate (zoom factor has been modified)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="82"/>
        <source>Add fave</source>
        <translation>Dodaj do ulubionych</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="84"/>
        <source>Reset parameters to default values</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="87"/>
        <source>Update filters (Ctrl+R / F5)</source>
        <translation>Uaktualnienie filtrów</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="89"/>
        <source>Rename fave</source>
        <translation>Zmiana nazwy ulubionego</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="91"/>
        <source>Remove fave</source>
        <translation>Usuń z ulubionych</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="95"/>
        <source>Expand/Collapse all</source>
        <translation>Rozwiń/zwiń wszystko</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="97"/>
        <source>G&apos;MIC (http://gmic.eu)&lt;br/&gt;GREYC (http://www.greyc.fr)&lt;br/&gt;CNRS (http://www.cnrs.fr)&lt;br/&gt;Normandy University (http://www.unicaen.fr)&lt;br/&gt;Ensicaen (http://www.ensicaen.fr)</source>
        <translation>G&apos;MIC (http://gmic.eu)&lt;br/&gt;GREYC (http://www.greyc.fr)&lt;br/&gt;CNRS (http://www.cnrs.fr)&lt;br/&gt;Uniwerstytet Normandii (http://www.unicaen.fr)&lt;br/&gt;Ensicaen (http://www.ensicaen.fr)</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="263"/>
        <source>Update completed</source>
        <translation>Aktualizacja ukończona</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="263"/>
        <location filename="../src/MainWindow.cpp" line="265"/>
        <location filename="../src/MainWindow.cpp" line="316"/>
        <source>Filter definitions have been updated.</source>
        <translation>Filtry zaktualizowane.</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="268"/>
        <source>No download was needed.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="447"/>
        <source>The update could not be achieved&lt;br&gt;because of the following errors:&lt;br&gt;</source>
        <translation>Aktualizacja nie powiodła się&lt;br/&gt;z następujących powodów:&lt;br/&gt;</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="453"/>
        <source>Update error</source>
        <translation>Błąd aktualizacji</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="596"/>
        <source>Error</source>
        <translation>Wystąpił błąd</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="675"/>
        <source>Waiting for cancelled jobs...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1004"/>
        <source>Import faves</source>
        <translation>Importuj ulubione</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1004"/>
        <source>Do you want to import faves from file below?&lt;br/&gt;%1</source>
        <translation>Czy chcesz importować ulubione z pliku poniżej?&lt;br/&gt;%1</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1007"/>
        <source>Don&apos;t ask again</source>
        <translation>Nie pytaj ponownie</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1096"/>
        <source>Confirmation</source>
        <translation>Potwierdż</translation>
    </message>
    <message>
        <location filename="../src/MainWindow.cpp" line="1096"/>
        <source>A gmic command is running.&lt;br&gt;Do you really want to close the plugin?</source>
        <translation>Polecenie gmic jest uruchomione.&lt;br&gt;Czy naprawdę chcesz zamknąć wtyczkę?</translation>
    </message>
</context>
<context>
    <name>MultilineTextParameterWidget</name>
    <message>
        <location filename="../ui/multilinetextparameterwidget.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/multilinetextparameterwidget.ui" line="29"/>
        <source>Update</source>
        <translation>Uaktualnij</translation>
    </message>
    <message>
        <location filename="../src/FilterParameters/MultilineTextParameterWidget.cpp" line="38"/>
        <source>Ctrl+Return</source>
        <translation>Ctrl+Enter</translation>
    </message>
</context>
<context>
    <name>ProgressInfoWidget</name>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="48"/>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="46"/>
        <source>Abort</source>
        <translation>Anuluj</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowidget.ui" line="55"/>
        <source>TextLabel</source>
        <translation>TextLabel</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="43"/>
        <source>G&apos;MIC-Qt Plug-in progression</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="141"/>
        <source>Updating filters...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="196"/>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="209"/>
        <source>[Processing %1 | %2]</source>
        <translation>[Przetwarzanie %1 | %2]</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWidget.cpp" line="211"/>
        <source>[Processing %1]</source>
        <translation>[Przetwarzanie %1]</translation>
    </message>
</context>
<context>
    <name>ProgressInfoWindow</name>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="14"/>
        <source>MainWindow</source>
        <translation>MainWindow</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="30"/>
        <location filename="../ui/progressinfowindow.ui" line="50"/>
        <source>TextLabel</source>
        <translation>TextLabel</translation>
    </message>
    <message>
        <location filename="../ui/progressinfowindow.ui" line="72"/>
        <source>Cancel</source>
        <translation>Anuluj</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="103"/>
        <source>%1 seconds</source>
        <translation>%1 sekund</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="113"/>
        <source>[Processing %1 | %2]</source>
        <translation>[Przetwarzanie %1 | %2]</translation>
    </message>
    <message>
        <location filename="../src/Widgets/ProgressInfoWindow.cpp" line="115"/>
        <source>[Processing %1]</source>
        <translation>[Przetwarzanie %1]</translation>
    </message>
</context>
<context>
    <name>QObject</name>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="81"/>
        <source>Select an image to open...</source>
        <translation>Wybierz obraz...</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="81"/>
        <source>PNG &amp; JPG files (*.png *.jpeg *.jpg *.PNG *.JPEG *.JPG)</source>
        <translation>PNG- i JPG- pliki (*.png *.jpeg *.jpg *.PNG *.JPEG *.JPG)</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="87"/>
        <source>Error</source>
        <translation>Wystąpił błąd</translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="87"/>
        <source>Could not open file.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/Host/None/host_none.cpp" line="91"/>
        <source>Default image</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersView/FiltersView.cpp" line="87"/>
        <source>Visible</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/FilterSelector/FiltersPresenter.cpp" line="85"/>
        <source>Available filters (%1)</source>
        <translation>Dostępne filtry (%1)</translation>
    </message>
</context>
<context>
    <name>SearchFieldWidget</name>
    <message>
        <location filename="../ui/SearchFieldWidget.ui" line="20"/>
        <source>Frame</source>
        <translation>Ramka</translation>
    </message>
    <message>
        <location filename="../src/Widgets/SearchFieldWidget.cpp" line="72"/>
        <source>Search</source>
        <translation>Wyszukiwanie</translation>
    </message>
    <message>
        <location filename="../src/Widgets/SearchFieldWidget.cpp" line="73"/>
        <source>Search in filters list (%1)</source>
        <translation>Szukaj w liście filtrów (%1)</translation>
    </message>
</context>
<context>
    <name>Updater</name>
    <message>
        <location filename="../src/Updater.cpp" line="206"/>
        <source>Could not read/decompress %1</source>
        <translation>Błąd odczytu/dekompresji %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="197"/>
        <source>Error downloading %1 (empty file?)</source>
        <translation>Błąd pobierania %1 (Pusty plik?)</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="212"/>
        <source>Error creating file %1</source>
        <translation>Błąd tworzenia pliku %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="216"/>
        <source>Error writing file %1</source>
        <translation>Błąd zapisu pliku %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="228"/>
        <source>Error downloading %1</source>
        <translation>Błąd pobierania %1</translation>
    </message>
    <message>
        <location filename="../src/Updater.cpp" line="256"/>
        <source>Download timeout: %1</source>
        <translation>Limit czasu pobierania: %1</translation>
    </message>
</context>
<context>
    <name>ZoomLevelSelector</name>
    <message>
        <location filename="../ui/zoomlevelselector.ui" line="14"/>
        <source>Form</source>
        <translation>GMIC</translation>
    </message>
</context>
</TS>
