/** -*- mode: c++ ; c-basic-offset: 2 -*-
 *
 *  @file ProgressInfoWidget.h
 *
 *  Copyright 2017 Sebastien Fourey
 *
 *  This file is part of G'MIC-Qt, a generic plug-in for raster graphics
 *  editors, offering hundreds of filters thanks to the underlying G'MIC
 *  image processing framework.
 *
 *  gmic_qt is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  gmic_qt is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with gmic_qt.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _GMIC_QT_PROGRESSINFOWIDGET_H_
#define _GMIC_QT_PROGRESSINFOWIDGET_H_
#include <QMainWindow>
#include <QTimer>

class GmicProcessor;
namespace Ui
{
class ProgressInfoWidget;
}

class ProgressInfoWidget : public QWidget {
  Q_OBJECT

public:
  explicit ProgressInfoWidget(QWidget * parent = 0);
  ~ProgressInfoWidget();

  enum Mode
  {
    GmicProcessingMode,
    FiltersUpdateMode
  };

  Mode mode() const;

  bool hasBeenCanceled() const;
  void setGmicProcessor(const GmicProcessor * processor);

public slots:
  void onTimeOut();
  void onCancelClicked();
  void stopAnimationAndHide();
  void startFilterThreadAnimationAndShow(bool showCancelButton);
  void startFiltersUpdateAnimationAndShow();
  void showBusyIndicator();
signals:
  void cancel();

private:
  void updateThreadInformation();
  void updateUpdateProgression();

  Ui::ProgressInfoWidget * ui;
  const GmicProcessor * _gmicProcessor;
  QTimer _timer;
  QTimer _showingTimer;
  Mode _mode;
  bool _canceled;
  bool _growing;
  static const int AnimationStep = 10;
};

#endif // _GMIC_QT_PROGRESSINFOWIDGET_H_
